//=============================================================================
//  MuseScore
//  Music Composition & Notation
// 
//  noteshaper.qml
//  MuseScore plugin based on "Shape notes - Sacred Harp & Aiken" by lasconic.
//  Original plugin Copyright (C) 2015 Nicolas Froment, GNU GPL v2
//  https://musescore.org/en/project/shapenotes
//
//  Edited to eliminate plugin's user interface and use a menu item only.
//  Karlin High 2015-08-07, same license terms as original.
//=============================================================================

import QtQuick 2.2
import MuseScore 1.0

MuseScore {
      version:  "1.0"
      description: qsTr("Change noteheads to shaped notes.")
      menuPath: "Plugins.Shaped Notes"

      property variant scales :  ['C', 'G', 'D', 'A', 'E', 'B', 'F', 'C', 'G', 'D', 'A', 'E', 'B', 'F', 'C'];

      // Apply the given function to all notes in selection
      // or, if nothing is selected, in the entire score

      function applyToNotesInSelection(func) {
            var cursor = curScore.newCursor();
            cursor.rewind(1);
            var startStaff;
            var endStaff;
            var endTick;
            var fullScore = false;
            if (!cursor.segment) { // no selection
                  fullScore = true;
                  startStaff = 0; // start with 1st staff
                  endStaff = curScore.nstaves - 1; // and end with last
            } else {
                  startStaff = cursor.staffIdx;
                  cursor.rewind(2);
                  if (cursor.tick == 0) {
                        // this happens when the selection includes
                        // the last measure of the score.
                        // rewind(2) goes behind the last segment (where
                        // there's none) and sets tick=0
                        endTick = curScore.lastSegment.tick + 1;
                  } else {
                        endTick = cursor.tick;
                  }
                  endStaff = cursor.staffIdx;
            }
            console.log(startStaff + " - " + endStaff + " - " + endTick)
            for (var staff = startStaff; staff <= endStaff; staff++) {
                  for (var voice = 0; voice < 4; voice++) {
                        cursor.rewind(1); // sets voice to 0
                        cursor.voice = voice; //voice has to be set after goTo
                        cursor.staffIdx = staff;

                        if (fullScore)
                              cursor.rewind(0) // if no selection, beginning of score

                        while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                              if (cursor.element && cursor.element.type == Element.CHORD) {
                                    var graceChords = cursor.element.graceNotes;
                                    for (var i = 0; i < graceChords.length; i++) {
                                          // iterate through all grace chords
                                          var notes = graceChords[i].notes;
                                          func(note);
                                    }
                                    var notes = cursor.element.notes;
                                    for (var i = 0; i < notes.length; i++) {
                                          var note = notes[i];
                                          func(note);
                                    }
                              }
                              cursor.next();
                        }
                  }
            }
      }

      function shapeNotes(note) {
        console.log("shapeNotes")
          var tpcNames = "FCGDAEB"
          var name = tpcNames[(note.tpc + 1) % 7]

          var names = "CDEFGAB"
          var curKey = curScore.keysig;
          var scale = scales[curKey+7];
          
          // Aiken-style 7-shape notes  
          //var degrees = [7, 8, 4, 9, 12, 10, 11]; // 7 notes
          
          // Uncomment this for Sacred Harp / fasola 4-shape notes  
          var degrees =[9, 12, 10, 9, 12, 10, 4]; // 4 notes
   
          // Uncomment this for unshaped notes  
          // var degrees =[0, 0, 0, 0, 0, 0, 0]; // unshaped notes

          note.headGroup = degrees[(names.indexOf(name) - names.indexOf(scale) +28)%7];           
      }

      onRun: {
            if (typeof curScore === 'undefined')
                  Qt.quit();
            console.log("hello 4-shapeNotes");
            applyToNotesInSelection(shapeNotes)            
         }
}
